import operator
import xml.etree.ElementTree as ETree

from ucca import core, layer0, layer1, convert

"""Utilities for tests."""


def empty():
    p = core.Passage(ID="1")
    layer0.Layer0(p)
    layer1.Layer1(p)
    return p


def basic():
    """Creates a basic :class:Passage to tinker with.

    Passage structure is as follows:
        Layer1: order by ID, heads = [1.2], all = [1.1, 1.2, 1.3]
        Layer2: order by node unique ID descending,
                heads = all = [2.2, 2.1], attrib={"test": True}
        Nodes (tag):
            1.1 (1)
            1.3 (3), attrib={"node": True}
            1.2 (x), order by edge tag
                children: 1.3 Edge: tag=test1, attrib={"Edge": True}
                          1.1 Edge: tag=test2
            2.1 (2), children [1.1, 1.2] with edge tags [test, test2]
            2.2 (2), children [1.1, 1.2, 1.3] with tags [test, test1, test]

    """
    p = core.Passage(ID="1")
    core.Layer(ID="1", root=p)
    core.Layer(ID="2", root=p, attrib={"test": True},
               orderkey=lambda x: -1 * int(x.ID.split(".")[1]))

    # Order is explicitly different in order to break the alignment between
    # the ID/Edge ordering and the order of creation/addition
    node11 = core.Node(ID="1.1", root=p, tag="1")
    node13 = core.Node(ID="1.3", root=p, tag="3", attrib={"node": True})
    node12 = core.Node(ID="1.2", root=p, tag="x",
                       orderkey=operator.attrgetter("tag"))
    node21 = core.Node(ID="2.1", root=p, tag="2")
    node22 = core.Node(ID="2.2", root=p, tag="2")
    node12.add("test2", node11)
    node12.add("test1", node13, edge_attrib={"edge": True})
    node21.add("test2", node12)
    node21.add("test", node11)
    node22.add("test1", node12)
    node22.add("test", node13)
    node22.add("test", node11)
    return p


def l1_passage():
    """Creates a Passage to work with using layer1 objects.

    Annotation layout (what annotation each terminal has):
        1: Linker, linked with the first parallel scene
        2-10: Parallel scene #1, 2-5 ==> Participant #1
            6-9 ==> Process #1, 10 ==> Punctuation, remote Participant is
            Adverbial #2
        11-19: Parallel scene #23, which encapsulated 2 scenes and a linker
            (not a real scene, has no process, only for grouping)
        11-15: Parallel scene #2 (under #23), 11-14 ==> Participant #3,
            15 ==> Adverbial #2, remote Process is Process #1
        16: Linker #2, links Parallel scenes #2 and #3
        17-19: Parallel scene #3, 17-18 ==> Process #3,
            19 ==> Participant #3, implicit Participant
        20: Punctuation (under the head)

    """

    p = core.Passage("1")
    l0 = layer0.Layer0(p)
    l1 = layer1.Layer1(p)
    # 20 terminals (1-20), #10 and #20 are punctuation
    terms = [l0.add_terminal(text=str(i), punct=(i % 10 == 0))
             for i in range(1, 21)]

    # Linker #1 with terminal 1
    link1 = l1.add_fnode(None, layer1.EdgeTags.Linker)
    link1.add(layer1.EdgeTags.Terminal, terms[0])

    # Scene #1: [[2 3 4 5 P] [6 7 8 9 A] [10 U] H]
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1.add(layer1.EdgeTags.Terminal, terms[1])
    p1.add(layer1.EdgeTags.Terminal, terms[2])
    p1.add(layer1.EdgeTags.Terminal, terms[3])
    p1.add(layer1.EdgeTags.Terminal, terms[4])
    a1.add(layer1.EdgeTags.Terminal, terms[5])
    a1.add(layer1.EdgeTags.Terminal, terms[6])
    a1.add(layer1.EdgeTags.Terminal, terms[7])
    a1.add(layer1.EdgeTags.Terminal, terms[8])
    l1.add_punct(ps1, terms[9])

    # Scene #23: [[11 12 13 14 15 H] [16 L] [17 18 19 H] H]
    # Scene #2: [[11 12 13 14 P] [15 D]]
    ps23 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    ps2 = l1.add_fnode(ps23, layer1.EdgeTags.ParallelScene)
    a2 = l1.add_fnode(ps2, layer1.EdgeTags.Participant)
    a2.add(layer1.EdgeTags.Terminal, terms[10])
    a2.add(layer1.EdgeTags.Terminal, terms[11])
    a2.add(layer1.EdgeTags.Terminal, terms[12])
    a2.add(layer1.EdgeTags.Terminal, terms[13])
    d2 = l1.add_fnode(ps2, layer1.EdgeTags.Adverbial)
    d2.add(layer1.EdgeTags.Terminal, terms[14])

    # Linker #2: [16 L]
    link2 = l1.add_fnode(ps23, layer1.EdgeTags.Linker)
    link2.add(layer1.EdgeTags.Terminal, terms[15])

    # Scene #3: [[16 17 S] [18 A] (implicit participant) H]
    ps3 = l1.add_fnode(ps23, layer1.EdgeTags.ParallelScene)
    p3 = l1.add_fnode(ps3, layer1.EdgeTags.State)
    p3.add(layer1.EdgeTags.Terminal, terms[16])
    p3.add(layer1.EdgeTags.Terminal, terms[17])
    a3 = l1.add_fnode(ps3, layer1.EdgeTags.Participant)
    a3.add(layer1.EdgeTags.Terminal, terms[18])
    l1.add_fnode(ps3, layer1.EdgeTags.Participant, implicit=True)

    # Punctuation #20 - not under a scene
    l1.add_punct(None, terms[19])

    # adding remote argument to scene #1, remote process to scene #2
    # creating linkages L1->H1, H2<-L2->H3
    l1.add_remote(ps1, layer1.EdgeTags.Participant, d2)
    l1.add_remote(ps2, layer1.EdgeTags.Process, p1)
    l1.add_linkage(link1, ps1)
    l1.add_linkage(link2, ps2, ps3)

    return p


def multi_sent():
    """Creates a :class:Passage with multiple sentences and paragraphs.

    Passage: [1 2 [3 P] H] . [[5 6 . P] H]
             [[8 P] . 10 . H]

    """
    p = core.Passage("1")
    l0 = layer0.Layer0(p)
    l1 = layer1.Layer1(p)
    terms = [l0.add_terminal(str(i), False) for i in range(1, 4)]
    terms.append(l0.add_terminal(".", True))
    terms.append(l0.add_terminal("5", False))
    terms.append(l0.add_terminal("6", False))
    terms.append(l0.add_terminal(".", True))
    terms.append(l0.add_terminal("8", False, paragraph=2))
    terms.append(l0.add_terminal(".", True, paragraph=2))
    terms.append(l0.add_terminal("10", False, paragraph=2))
    terms.append(l0.add_terminal(".", True, paragraph=2))
    h1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    h2 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    h3 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    p1 = l1.add_fnode(h1, layer1.EdgeTags.Process)
    p2 = l1.add_fnode(h2, layer1.EdgeTags.Process)
    p3 = l1.add_fnode(h3, layer1.EdgeTags.Process)
    h1.add(layer1.EdgeTags.Terminal, terms[0])
    h1.add(layer1.EdgeTags.Terminal, terms[1])
    p1.add(layer1.EdgeTags.Terminal, terms[2])
    l1.add_punct(None, terms[3])
    p2.add(layer1.EdgeTags.Terminal, terms[4])
    p2.add(layer1.EdgeTags.Terminal, terms[5])
    l1.add_punct(p2, terms[6])
    p3.add(layer1.EdgeTags.Terminal, terms[7])
    l1.add_punct(h3, terms[8])
    h3.add(layer1.EdgeTags.Terminal, terms[9])
    l1.add_punct(h3, terms[10])
    return p


def crossing():
    """Creates a :class:Passage with multiple sentences and paragraphs, with crossing edges.

    Passage: [1 2 [3 P(remote)] H] .
             [[3 P] . 4 . H]

    """
    p = core.Passage("1")
    l0 = layer0.Layer0(p)
    l1 = layer1.Layer1(p)
    terms = [
        l0.add_terminal("1", False),
        l0.add_terminal("2", False),
        l0.add_terminal(".", True),
        l0.add_terminal("3", False, paragraph=2),
        l0.add_terminal(".", True, paragraph=2),
        l0.add_terminal("4", False, paragraph=2),
        l0.add_terminal(".", True, paragraph=2),
    ]
    h1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    h2 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    p1 = l1.add_fnode(h2, layer1.EdgeTags.Process)
    l1.add_remote(h1, layer1.EdgeTags.Process, p1)
    h1.add(layer1.EdgeTags.Terminal, terms[0])
    h1.add(layer1.EdgeTags.Terminal, terms[1])
    l1.add_punct(None, terms[2])
    p1.add(layer1.EdgeTags.Terminal, terms[3])
    l1.add_punct(h2, terms[4])
    h2.add(layer1.EdgeTags.Terminal, terms[5])
    l1.add_punct(h2, terms[6])
    return p


def discontiguous():
    """Creates a highly-discontiguous Passage object."""
    p = core.Passage("1")
    l0 = layer0.Layer0(p)
    l1 = layer1.Layer1(p)
    # 20 terminals (1-20), #10 and #20 are punctuation
    terms = [l0.add_terminal(text=str(i), punct=(i % 10 == 0))
             for i in range(1, 21)]

    # First parallel scene, stretching on terminals 1-10
    # The dashed edge tags (e.g. -C, C-) mean discontiguous units
    # [PS [D [E 0] [C- 1] [E 2] [-C 3]]
    #     [A- 4] [P- 5 6] [-A 7] [F 8] [-P [U 9]]]
    # In addition, D takes P as a remote G
    ps1 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    d1 = l1.add_fnode(ps1, layer1.EdgeTags.Adverbial)
    e1 = l1.add_fnode(d1, layer1.EdgeTags.Elaborator)
    c1 = l1.add_fnode(d1, layer1.EdgeTags.Center)
    e2 = l1.add_fnode(d1, layer1.EdgeTags.Elaborator)
    a1 = l1.add_fnode(ps1, layer1.EdgeTags.Participant)
    p1 = l1.add_fnode(ps1, layer1.EdgeTags.Process)
    f1 = l1.add_fnode(ps1, layer1.EdgeTags.Function)
    l1.add_remote(d1, layer1.EdgeTags.Ground, p1)
    e1.add(layer1.EdgeTags.Terminal, terms[0])
    c1.add(layer1.EdgeTags.Terminal, terms[1])
    e2.add(layer1.EdgeTags.Terminal, terms[2])
    c1.add(layer1.EdgeTags.Terminal, terms[3])
    a1.add(layer1.EdgeTags.Terminal, terms[4])
    p1.add(layer1.EdgeTags.Terminal, terms[5])
    p1.add(layer1.EdgeTags.Terminal, terms[6])
    a1.add(layer1.EdgeTags.Terminal, terms[7])
    f1.add(layer1.EdgeTags.Terminal, terms[8])
    l1.add_punct(p1, terms[9])

    # Second parallel scene, stretching on terminals 11-14 + 18-20
    # [PS- [D IMPLICIT] [G IMPLICIT] [P 10 11 12 13]]
    # [-PS [A 17 18 [U 19]]]
    ps2 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    l1.add_fnode(ps2, layer1.EdgeTags.Adverbial, implicit=True)
    l1.add_fnode(ps2, layer1.EdgeTags.Ground, implicit=True)
    p2 = l1.add_fnode(ps2, layer1.EdgeTags.Process)
    a2 = l1.add_fnode(ps2, layer1.EdgeTags.Participant)
    p2.add(layer1.EdgeTags.Terminal, terms[10])
    p2.add(layer1.EdgeTags.Terminal, terms[11])
    p2.add(layer1.EdgeTags.Terminal, terms[12])
    p2.add(layer1.EdgeTags.Terminal, terms[13])
    a2.add(layer1.EdgeTags.Terminal, terms[17])
    a2.add(layer1.EdgeTags.Terminal, terms[18])
    l1.add_punct(a2, terms[19])

    # Third parallel scene, stretching on terminals 15-17
    # [PS [P IMPLICIT] 14 [A 15 16]]
    ps3 = l1.add_fnode(None, layer1.EdgeTags.ParallelScene)
    ps3.add(layer1.EdgeTags.Terminal, terms[14])
    l1.add_fnode(ps3, layer1.EdgeTags.Process, implicit=True)
    a3 = l1.add_fnode(ps3, layer1.EdgeTags.Participant)
    a3.add(layer1.EdgeTags.Terminal, terms[15])
    a3.add(layer1.EdgeTags.Terminal, terms[16])

    return p


def loaded():
    return convert.from_standard(load_xml("test_files/standard3.xml"))


def loaded_valid():
    return convert.from_standard(load_xml("test_files/standard3_valid.xml"))


def load_xml(path):
    """XML file path ==> root element
    :param path: path to XML file
    """
    with open(path, encoding="utf-8") as f:
        return ETree.ElementTree().parse(f)


PASSAGES = (loaded, loaded_valid, multi_sent, crossing, discontiguous, l1_passage, empty)


def create_passage(num_terms=3, *punct):
    p = core.Passage("1")
    l0 = layer0.Layer0(p)
    l1 = layer1.Layer1(p)
    terms = [l0.add_terminal(text=str(i), punct=(i in punct)) for i in range(1, num_terms + 1)]
    return p, l1, terms


def attach_terminals(terms, *nodes):
    for term, node in zip(terms, nodes):
        node.add(layer1.EdgeTags.Terminal, term)



#!/usr/bin/env python3
import argparse
import sys

from uccaapp.api import ServerAccessor

desc = """Convert a passage file to JSON format and upload to UCCA-App as a completed task"""


class AnnotationTaskCreator(ServerAccessor):
    def __init__(self, user_id, **kwargs):
        super().__init__(**kwargs)
        self.set_user(user_id)

    def create_task(self, filename, **kwargs):
        del kwargs
        with open(filename) as f:
            num = 0
            for line in f:
                fields = line.strip().split()
                if len(fields) != 2:
                    sys.stderr.write("Error in line: "+line.strip())
                    continue
                user_id = fields[0]
                user_model = self.get_user(user_id)
                tok_task_out = self.get_task(fields[1])
                task_in = dict(type="ANNOTATION", status="SUBMITTED",
                               project=self.project, user=user_model,
                               passage=tok_task_out["passage"], manager_comment="Reviews corpus",
                               user_comment="Test", parent=tok_task_out,
                               is_demo=False, is_active=True)
                self.create_annotation_task(**task_in)
                num += 1
            print("Uploaded %d tasks successfully." % num, file=sys.stderr)

    @staticmethod
    def add_arguments(argparser):
        argparser.add_argument("filename", help="a file where each line is a <User ID> <TOKENIZATION TASK ID>")
        ServerAccessor.add_user_id_argument(argparser)
        ServerAccessor.add_arguments(argparser)


def main(**kwargs):
    AnnotationTaskCreator(**kwargs).create_task(**kwargs)


if __name__ == "__main__":
    argument_parser = argparse.ArgumentParser(description=desc)
    AnnotationTaskCreator.add_arguments(argument_parser)
    main(**vars(argument_parser.parse_args()))
    sys.exit(0)



import sys
from itertools import islice

import argparse
from multiprocessing import Pool

from ucca.ioutil import get_passages_with_progress_bar
from ucca.normalization import normalize
from ucca.validation import validate


class Validator:
    def __init__(self, normalization=False, extra=False, linkage=True):
        self.normalization = normalization
        self.extra = extra
        self.linkage = linkage

    def validate_passage(self, passage):
        if self.normalization:
            normalize(passage, extra=self.extra)
        return passage.ID, list(validate(passage, linkage=self.linkage))


def main(args):
    with Pool(10) as pool:
        errors = pool.map(Validator(args.normalize, args.extra, linkage=args.linkage).validate_passage,
                          get_passages_with_progress_bar(args.filenames, desc="Validating", converters={}))
    errors = dict(islice(((k, v) for k, v in errors if v), 1 if args.strict else None))
    if errors:
        id_len = max(map(len, errors))
        for passage_id, es in sorted(errors.items()):
            for i, e in enumerate(es):
                print("%-*s|%s" % (id_len, "" if i else passage_id, e))
        sys.exit(1)
    else:
        print("No errors found.")


def check_args(parser, args):
    if args.extra and not args.normalize:
        parser.error("Cannot specify --extra without --normalize")
    return args


if __name__ == "__main__":
    argparser = argparse.ArgumentParser(description="Validate UCCA passages")
    argparser.add_argument("filenames", nargs="+", help="files or directories to validate")
    argparser.add_argument("-S", "--strict", action="store_true", help="fail as soon as a violation is found")
    argparser.add_argument("-n", "--normalize", action="store_true", help="normalize before validation")
    argparser.add_argument("-e", "--extra", action="store_true", help="extra normalization rules")
    argparser.add_argument("--no-linkage", dest="linkage", action="store_false", help="skip linkage validations")
    main(check_args(argparser, argparser.parse_args()))



import os
from argparse import ArgumentParser
from xml.etree.ElementTree import tostring

from scripts.ucca_db2 import get_by_xids, get_most_recent_passage_by_uid
from ucca import convert
from ucca.ioutil import write_passage

desc = "Download passages to old UCCA annotation app"


def download_passage(xid, **kwargs):
    return get_by_xids(xids=[xid], **kwargs)[0]


def download_most_recent_passage_by_uid(uid, passage_id, **kwargs):
    return get_most_recent_passage_by_uid(uid, passage_id, **kwargs)


def main(args):
    os.makedirs(args.outdir, exist_ok=True)
    kwargs = dict(db_name=args.db_name, host_name=args.host_name)
    with open(args.filename, encoding="utf-8") as f:
        for passage_id, id_field in map(str.split, f):
            if args.verbose:
                print("Getting passage " + passage_id + " from user " + id_field)
            if args.method == "xid":
                xml_root = download_passage(id_field, **kwargs)
            elif args.method == "uid":
                xml_root = download_most_recent_passage_by_uid(id_field, passage_id, **kwargs)
            else:
                raise ValueError("Unknown method: '%s'" % args.method)
            if args.write_site:
                site_filename = passage_id + "_site_download.xml"
                with open(site_filename, "w", encoding="utf-8") as fsite:
                    print(tostring(xml_root).decode(), file=fsite)
                if args.verbose:
                    print("Wrote '%s'" % site_filename)
            write_passage(convert.from_site(xml_root), outdir=args.outdir, verbose=args.verbose)


if __name__ == "__main__":
    argparser = ArgumentParser(description=desc)
    argparser.add_argument("filename", help="specification filename with (passage ID, xid OR uid) per passage")
    argparser.add_argument("--method", required=True, choices=("xid", "uid"), help="Get by xid or latest by paid, uid")
    argparser.add_argument("-d", "--db-name", default="work", help="database name")
    argparser.add_argument("-H", "--host-name", default="pgserver", help="host name")
    argparser.add_argument("-o", "--outdir", default=".", help="directory to write created XML IDs to")
    argparser.add_argument("--write-site", action="store_true", help="write site format for debugging after download")
    argparser.add_argument("-v", "--verbose", action="store_true", help="print tagged text for each passage")
    main(argparser.parse_args())



import sys
from collections import Counter

import datetime
import psycopg2
import re
from xml.etree.ElementTree import fromstring

from ucca import convert, layer1
from ucca.layer1 import EdgeTags as ET
from ucca.layer1 import NodeTags as NT

UNK_LINKAGE_TYPE = 'UNK'
PLACE_HOLDER = "%s"


##############################################################################
# Returns the most recent xmls from db with a passage id pid and usernames
# (a list). The xmls are ordered in the same way as the list usernames.
##############################################################################


def get_connection(db, host):
    """ connects to the db and host, returns a connection object """
    return psycopg2.connect(host=host, database=db)


def get_cursor_con(con):
    """ gets a connection and create a cursur to the search path"""
    cur = con.cursor()
    cur.execute("SET search_path TO oabend")
    return cur


def get_cursor(db, host):
    """ create a cursur to the search path
        NOTE: if you have a connection up use get_cursor_con instead
    """
    return get_cursor_con(get_connection(db, host))


def get_xml_trees(db, host, pid, usernames, graceful=False):
    """ Params: db, host, paragraph id, the list of usernames wanted,
        Optional:
        graceful: True if no excpetions are to be raised
        excpetion raised if a user did not submit an annotation for the passage
        returns a list of xml roots elements
        """
    cur = get_cursor(db, host)
    xmls = []
    for username in usernames:
        username = str(username)  # precaution for cases bad input e.g. 101
        cur_uid = get_uid(db, host, username)
        cur.execute("SELECT xml FROM xmls WHERE paid=" + PLACE_HOLDER +
                    " AND uid=" + PLACE_HOLDER + "ORDER BY ts DESC",
                    (pid, cur_uid))
        raw_xml = cur.fetchone()
        if not raw_xml and not graceful:
            raise Exception("The user " + username +
                            " did not submit an annotation for this passage")
        else:
            xmls.append(fromstring(raw_xml[0]))
    return xmls


def get_by_xid(db, host, xid, graceful=False):
    """Returns the passages that correspond to the xid

        Optional:
        graceful: True if no excpetions are to be raised
        excpetion raised if xid does no exist
    """
    # precaution for bad input e.g. 101->'101'
    xid = str(xid)
    cur = get_cursor(db, host)
    cur.execute("SELECT xml FROM xmls WHERE id" + "=" +
                PLACE_HOLDER, (int(xid),))
    raw_xml = cur.fetchone()
    if not raw_xml and not graceful:
        raise Exception("The xid " + xid + " does not exist")
    elif raw_xml:
        return fromstring(raw_xml[0])


def get_by_xids(db, host, xids, graceful=False):
    """Returns the passages that correspond to iterable xids

        Optional:
        graceful: True if no excpetions are to be raised
        excpetion raised if xid does no exist
    """
    _ = get_cursor(db, host)
    xmls = [get_by_xid(db, host, xid, graceful) for xid in xids]
    return xmls


def get_uid(db, host, username, operator="=", graceful=False):
    """Returns the uid matching the given username.
        Optional:
        operator: default operator is '='
        graceful: True if no excpetions are to be raised
        excpetion raised if a user does no exist
        """
    cur = get_cursor(db, host)
    username = str(username)
    cur.execute("SELECT id FROM users WHERE username" + operator +
                PLACE_HOLDER, (username,))
    cur_uid = cur.fetchone()
    if not cur_uid and not graceful:
        raise Exception("The user " + username + " does not exist")
    return cur_uid[0]


def write_to_db(db, host, xml, new_pid, new_prid, username,
                operator="=", graceful=False):
    """writes to db
        Optional:
        operator: default operator is '='
        graceful: True if no excpetions are to be raised
        excpetion raised if a user does no exist
    """
    print("warning this function was not tested and better" +
          " API specification (parameters what is written) is needed")
    cur = get_cursor(db, host)
    cur_uid = get_uid(db, host, username, operator, graceful)
    now = datetime.datetime.now()
    cur.execute("INSERT INTO xmls VALUES (NULL, " +
                (PLACE_HOLDER + ', ') * 5 + "0, " +
                PLACE_HOLDER + ")", (xml, new_pid, new_prid, cur_uid, '', now))


def print_most_recent_xids(db, host, username, n=10):
    """print the most recent xids of the given username."""
    cur_uid = get_uid(db, host, username)
    cur = get_cursor(db, host)
    cur.execute("SELECT id, paid FROM xmls WHERE uid=" + PLACE_HOLDER +
                " ORDER BY ts DESC", (cur_uid,))
    print(username)
    print("=============")
    for xid in get_most_recent_xids(db, host, username, n):
        print(xid)


def get_most_recent_xids(db, host, username, n=10):
    """Returns the n most recent xids of the given username."""
    cur_uid = get_uid(db, host, username)
    cur = get_cursor(db, host)
    cur.execute("SELECT id, paid FROM xmls WHERE uid=" + PLACE_HOLDER +
                " ORDER BY ts DESC", (cur_uid,))
    r = [cur.fetchone() for _ in range(n)]
    return r


def get_passage(db, host, pid):
    """Returns the passages with the given id numbers"""
    cur = get_cursor(db, host)
    cur.execute("SELECT passage FROM passages WHERE id=" +
                PLACE_HOLDER, (pid,))
    output = cur.fetchone()
    if not output:
        raise Exception("No passage with ID=" + pid)
    return output[0]


def linkage_type(u):
    """
    Returns the type of the primary linkage
    the scene participates in.
    It can be A,E or H.
    If it is a C, it returns the tag of
    the first fparent which is an A,E or H.
    If it does not find an fparent
    with either of these categories,
    it returns UNK_LINKAGE_TYPE.
    """
    cur_u = u
    while cur_u:
        if cur_u.ftag in [ET.Participant, ET.Elaborator, ET.ParallelScene]:
            return cur_u.ftag
        elif cur_u.ftag != ET.Center:
            return UNK_LINKAGE_TYPE
        else:
            cur_u = cur_u.fparent
    return UNK_LINKAGE_TYPE


def unit_length(u):
    """
    Returns the number of terminals
    (excluding remote units and punctuations)
    that are descendants of the unit u.
    """
    return len(u.get_terminals(punct=False, remotes=False))


def get_tasks(db, host, username):
    """
    Returns for that user a list of submitted passages
    and a list of assigned but not submitted passages.
    Each passage is given in the format:
    (<passage ID>, <source>, <recent submitted xid or -1 if not submitted>,
    <number of tokens in the passage>,
     <number of units in the passage>, <number of scenes in the passage>,
    <average length of a scene>).
     It also returns a distribution of the categories.
    """
    output_submitted = []
    category_distribution = Counter()
    # the categories of scenes. can be A, E or H
    scene_distribution = Counter()

    uid = get_uid(db, username)
    cur = get_cursor(db, username)
    cur.execute("SELECT pid,status FROM tasks WHERE uid=" +
                PLACE_HOLDER, (uid,))
    r = cur.fetchall()
    submitted_paids = [x[0] for x in r if x[1] == 1]
    incomplete_paids = [x[0] for x in r if x[1] == 0]

    wspace = re.compile("\\s+")

    for paid in submitted_paids:
        sum_scene_length = 0
        if paid < 100:  # skipping training passages
            continue
        cur.execute("SELECT passage,source FROM passages WHERE id=" +
                    PLACE_HOLDER, (paid,))
        r = cur.fetchone()
        if r:
            num_tokens = len(wspace.split(r[0])) - 1
            source = r[1]
            cur.execute("SELECT id, xml FROM xmls WHERE paid=" +
                        PLACE_HOLDER + " AND uid=" + PLACE_HOLDER +
                        " AND status=" + PLACE_HOLDER +
                        " ORDER BY ts DESC", (paid, uid, 1))
            r = cur.fetchone()
            if r:
                xid = r[0]
                # noinspection PyBroadException
                try:
                    ucca_dag = convert.from_site(fromstring(r[1]))
                except Exception:
                    sys.stderr.write("Skipped.\n")
                    continue
                num_units = len([x for x in ucca_dag.layer(layer1.LAYER_ID).all
                                 if x.tag == NT.Foundational]) - 1
                for node in ucca_dag.layer(layer1.LAYER_ID).all:
                    category_distribution.update([e.tag for e in node
                                                  if e.tag
                                                  not in
                                                  [ET.Punctuation, ET.LinkArgument, ET.LinkRelation, ET.Terminal]])
                # getting the scene categories
                scenes = [x for x in ucca_dag.layer(layer1.LAYER_ID).all
                          if x.tag == NT.Foundational and x.is_scene()]
                scene_distribution.update([linkage_type(sc) for sc in scenes])
                sum_scene_length += sum([unit_length(x) for x in scenes])

        output_submitted.append((paid, source, xid,
                                 num_tokens, num_units, len(scenes),
                                 1.0 * sum_scene_length / len(scenes)))

    return output_submitted, category_distribution, scene_distribution



from argparse import ArgumentParser
from xml.etree.ElementTree import tostring

from scripts.ucca_db2 import CONNECTION, write_to_db
from ucca import convert
from ucca.ioutil import get_passages_with_progress_bar

desc = "Upload passages to old UCCA annotation app"


def upload_passage(xml_root, site_filename=None, verbose=False, **kwargs):
    decoded = tostring(xml_root).decode()
    if site_filename:
        with open(site_filename, "w", encoding="utf-8") as f:
            print(decoded, file=f)
        if verbose:
            print("Wrote '%s'" % site_filename)
    return write_to_db(xml=decoded, **kwargs)


def main(args):
    with open(args.out, "w", encoding="utf-8") as f:
        for passage in get_passages_with_progress_bar(args.passages):
            out = upload_passage(convert.to_site(passage), verbose=args.verbose,
                                 site_filename=passage.ID + "_site_upload.xml" if args.write_site else None,
                                 db_name=args.db_name, host_name=args.host_name,
                                 new_pid=passage.ID, new_prid=args.project_id, username=args.username)
            print(passage.ID, out, file=f)
            if args.verbose:
                print("Uploaded passage %s with xid=%s" % (passage.ID, out))
    if CONNECTION is not None:
        CONNECTION.commit()
    print("Wrote '%s'" % args.out)


if __name__ == "__main__":
    argparser = ArgumentParser(description=desc)
    argparser.add_argument("passages", nargs="+", help="the corpus, given as xml/pickle file names")
    argparser.add_argument("-d", "--db-name", default="work", help="database name")
    argparser.add_argument("-H", "--host-name", default="pgserver", help="host name")
    argparser.add_argument("-p", "--project-id", default="63", help="project ID")
    argparser.add_argument("-u", "--username", default="danielh", help="username")
    argparser.add_argument("-o", "--out", default="xids.txt", help="file to write created XML IDs to")
    argparser.add_argument("--write-site", action="store_true", help="write site format for debugging before upload")
    argparser.add_argument("-v", "--verbose", action="store_true", help="print tagged text for each passage")
    main(argparser.parse_args())



#!/usr/bin/env python3

import argparse

from ucca.textutil import get_word_vectors

desc = """Load word vectors file to make sure it works."""


def main(args):
    for filename in args.filenames:
        vectors, dim = get_word_vectors(size=args.rows, dim=args.dim, filename=filename)
        print("Loaded %d rows, dim=%d" % (len(vectors), dim))


if __name__ == '__main__':
    argparser = argparse.ArgumentParser(description=desc)
    argparser.add_argument("filenames", nargs="+", help="word vector files to load")
    argparser.add_argument("-r", "--rows", type=int, help="maximum number of word vectors")
    argparser.add_argument("-d", "--dim", type=int, help="maximum dimension of word vectors")
    main(argparser.parse_args())
